@@ -74,7 +74,7 @@ BEGIN {
     require Exporter;
 
     # set the version for version checking
-    our $VERSION   = 4.09;
+    our $VERSION   = 4.10;
     our @ISA       = qw(Exporter);
     our @EXPORT    = qw();
     our @EXPORT_OK = qw();
@@ -99,24 +99,26 @@ my $fb = Graphics::Framebuffer->new(parameter => value);
 
 =head3 PARAMETERS
 
- Emulation Mode Only!  Otherwise, parameters are ignored.
-
 =over 2
 
-=item B< VXRES>
+=item B< FB_DEVICE>
+
+ Framebuffer device name. Default is /dev/fb0
+
+=item B< VXRES (Emulation Mode Only!  Otherwise ignored.)>
 
  Width of the emulation framebuffer.  Default is 640.
 
-=item B< VYRES>
+=item B< VYRES (Emulation Mode Only!  Otherwise ignored.)>
 
  Height of the emulation framebuffer.  Default is 480.
 
-=item B< BITS>
+=item B< BITS (Emulation Mode Only!  Otherwise ignored.)>
 
  Number of bits per pixel in the emulation framebuffer.
  Default is 32.
 
-=item B< BYTES>
+=item B< BYTES (Emulation Mode Only!  Otherwise ignored.)>
 
  Number of bytes per pixel in the emulation framebuffer.
  It's best to keep it BITS/8.  Default is 4.
@@ -179,9 +181,10 @@ sub new {
         'BYTES'                    => 4,
         'XOFFSET'                  => 0,
         'YOFFSET'                  => 0,
+        'FB_DEVICE'                => '/dev/fb0',
         @_
     };
-    if (open($self->{'FB'}, '+</dev/fb0')) {
+    if (open($self->{'FB'}, '+<',$self->{'FB_DEVICE'})) {
         binmode($self->{'FB'});
 
         ($self->{'xres'}, $self->{'yres'}, $self->{'xres_virtual'}, $self->{'yres_virtual'}, $self->{'xoffset'}, $self->{'yoffset'}, $self->{'bits_per_pixel'}, $self->{'grayscale'}, $self->{'bitfields'}, $self->{'nonstd'}, $self->{'activate'}, $self->{'height'}, $self->{'width'}, $self->{'accel_flags'}, $self->{'pixclock'}, $self->{'left_margin'}, $self->{'right_margin'}, $self->{'upper_margin'}, $self->{'lower_margin'}, $self->{'hsync_len'}, $self->{'vsync_len'}, $self->{'sync'}, $self->{'vmode'}, @dummy) = _get_info($self->{'FBIOGET_VSCREENINFO'}, $self->{'FBioget_vscreeninfo'}, $self->{'FB'});
@@ -288,6 +291,91 @@ sub draw_mode {
     }
 }
 
+=head2 normal_mode
+
+This is an alias to draw_mode($fb->{'NORMAL_MODE'})
+
+=over 1
+
+ $fb->normal_mode();
+
+=back
+
+=cut
+
+sub normal_mode {
+    my $self = shift;
+    $self->draw_mode($self->{'NORMAL_MODE'});
+}
+
+=head2 xor_mode
+
+This is an alias to draw_mode($fb->{'XOR_MODE'})
+
+=over 1
+
+ $fb->xor_mode();
+
+=back
+
+=cut
+
+sub xor_mode {
+    my $self = shift;
+    $self->draw_mode($self->{'XOR_MODE'});
+}
+
+=head2 or_mode
+
+This is an alias to draw_mode($fb->{'OR_MODE'})
+
+=over 1
+
+ $fb->or_mode();
+
+=back
+
+=cut
+
+sub or_mode {
+    my $self = shift;
+    $self->draw_mode($self->{'OR_MODE'});
+}
+
+=head2 and_mode
+
+This is an alias to draw_mode($fb->{'AND_MODE'})
+
+=over 1
+
+ $fb->and_mode();
+
+=back
+
+=cut
+
+sub and_mode {
+    my $self = shift;
+    $self->draw_mode($self->{'AND_MODE'});
+}
+
+=head2 mask_mode
+
+This is an alias to draw_mode($fb->{'MASK_MODE'})
+
+=over 1
+
+ $fb->mask_mode();
+
+=back
+
+=cut
+
+sub mask_mode {
+    my $self = shift;
+    $self->draw_mode($self->{'MASK_MODE'});
+}
+
 =head2 clear_screen
 
 Fills the entire screen with the background color
@@ -726,22 +814,22 @@ sub ellipse {
     my $cy      = int($params->{'y'});
     my $XRadius = int($params->{'xradius'});
     my $YRadius = int($params->{'yradius'});
+    $XRadius    = 1 if ($XRadius < 1);
+    $YRadius    = 1 if ($YRadius < 1);
 
-    $XRadius = 1 if ($XRadius < 1);
-    $YRadius = 1 if ($YRadius < 1);
     my $filled = int($params->{'filled'});
     my $fact   = $params->{'factor'} || 0;
     my $size   = $params->{'pixel_size'} || 1;
-    $size = 1 if ($filled);
+    $size      = 1 if ($filled);
 
     my ($old_cyy, $old_cy_y);
     if ($fact == 0) {
         $fact = 1;
     }
-    my $TwoASquare = (2 * ($XRadius * $XRadius)) * $fact;
-    my $TwoBSquare = (2 * ($YRadius * $YRadius)) * $fact;
-    my $x          = $XRadius;
-    my $y          = 0;
+    my $TwoASquare   = (2 * ($XRadius * $XRadius)) * $fact;
+    my $TwoBSquare   = (2 * ($YRadius * $YRadius)) * $fact;
+    my $x            = $XRadius;
+    my $y            = 0;
     my $XChange      = ($YRadius * $YRadius) * (1 - (2 * $XRadius));
     my $YChange      = ($XRadius * $XRadius);
     my $EllipseError = 0;
@@ -949,11 +1037,11 @@ It draws a frame if 'filled' is 0 or a filled box if 'filled' is 1.
 =over 1
 
  $fb->rbox({
-    'x' => 100,
-    'y' => 100,
-    'width' => 200,
-    'height' => 150,
-    'filled' => 0,
+    'x'          => 100,
+    'y'          => 100,
+    'width'      => 200,
+    'height'     => 150,
+    'filled'     => 0,
     'pixel_size' => 2
  });
 
@@ -970,9 +1058,9 @@ sub rbox {
     my $h      = int($params->{'height'});
     my $filled = int($params->{'filled'} || 0);
     my $size   = int($params->{'pixel_size'} || 1);
-    $size = 1 if ($filled);
-    my $xx = $x + $w;
-    my $yy = $y + $h;
+    $size      = 1 if ($filled);
+    my $xx     = $x + $w;
+    my $yy     = $y + $h;
     $self->box({'x' => $x, 'y' => $y, 'xx' => $xx, 'yy' => $yy, 'filled' => $filled, 'pixel_size' => $size});
 } ## end sub rbox
 
@@ -1039,7 +1127,7 @@ sub set_b_color {
         $G = int($G / 8);
         $B = int($B / 8);
         $self->{'B_COLOR'} = ($R << 11) + ($G << 6) + $B;
-        $self->{'B_COLOR'} = pack('S', $self->{'COLOR'});
+        $self->{'B_COLOR'} = pack('S', $self->{'B_COLOR'});
     }
 } ## end sub set_b_color
 
@@ -1096,7 +1184,7 @@ sub pixel {
 Does a flood fill starting at point x,y.  It samples the color
 at that point and determines that color to be the "background"
 color, and proceeds to fill in, with the current global color,
-until the "backround" color is replaced with the new color.
+until the "background" color is replaced with the new color.
 Clipping applies.
 
 BECAUSE OF ITS RECURSIVE NATURE, IT CAN CHOW DOWN ON MEMORY
@@ -1109,6 +1197,13 @@ running!  This is a stack issue.
  $fb->fill({'x' => 334, 'y' => 23});
 
 =back
+
+** Perhaps the memory issue could be avoided if somehow the flood fill would
+   operate in blocks, instead of recurse until it can find no more to fill.
+   My brain hurts thinking about it.
+
+   Maybe just a different fill algorithmn might be better.
+
 =cut
 
 sub fill {
@@ -1306,7 +1401,7 @@ sub blit_write {
     $h = $self->{'YRES'}         if ($h > $self->{'YRES'});
     $w = $self->{'XX_CLIP'} - $x if (($x + $w) > $self->{'XX_CLIP'});
     my $scan = $w * $self->{'BYTES'};
-    my $WW;
+
     my $yend = $y + $h;
     if ($yend > $self->{'YY_CLIP'}) {
         $yend = $self->{'YY_CLIP'};
@@ -1480,6 +1575,8 @@ This is best called twice, first in bounding box mode, and then in normal mode.
 
 sub ttf_print {
 
+    # Yes, this is a "hack".                                                     #
+
     # This uses the 'Imager' package.  It allocates a temporary screen buffer    #
     # and prints to it, then this buffer is dumped to the screen at the x,y      #
     # coordinates given.  Since no decent True Type packages or libraries are    #
@@ -1683,8 +1780,12 @@ sub RGB_to_16 {
     ##############################################################################
     ##                               RGB to 16 Bit                              ##
     ##############################################################################
-    # Converts a 32 bit pixel value to a 16 bit pixel value.                     #
+    # Converts a 24 bit pixel value to a 16 bit pixel value.                     #
+    # -------------------------------------------------------------------------- #
+    # This is not a fancy table based color conversion.  This merely uses math,  #
+    # and thus the quality is lacking on the output.                             #
     ##############################################################################
+
     my $self   = shift;
     my $params = shift;
 
@@ -1710,6 +1811,15 @@ Converts 32 bit color values to 16 bit
 =cut
 
 sub RGBA_to_16 {
+    ##############################################################################
+    ##                              RGBA to 16 Bit                              ##
+    ##############################################################################
+    # Converts a 32 bit pixel value to a 16 bit pixel value.                     #
+    # -------------------------------------------------------------------------- #
+    # This is not a fancy table based color conversion.  This merely uses math,  #
+    # and thus the quality is lacking on the output.  This discards the alpha    #
+    # channel.                                                                   #
+    ##############################################################################
     my $self   = shift;
     my $params = shift;
 
@@ -1790,14 +1900,23 @@ __END__
 
 Richard Kelsch <rich@rk-internet.com>
 
-Copyright 2013 Richard Kelsch, All Rights Reserved.
+=head1 COPYRIGHT
+
+Copyright 2013-2015 Richard Kelsch, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-Version 4.09 (November 26, 2014)
+Version 4.10 (May 10, 2015)
+
+=head1 THANKS
+
+My thanks go out to those using this module and submitting helpful patches
+and suggestions for improvement:
+
+Markus Maier
 
 =cut
 
